package com.codigoparallevar.minicards.bridge;

import android.app.Service;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.IBinder;
import android.os.Process;
import android.util.Log;
import android.widget.Toast;

import com.codigoparallevar.minicards.ConfigManager;
import com.codigoparallevar.minicards.types.functional.Tuple2;
import com.codigoparallevar.minicards.ui_helpers.DoAsync;
import com.programaker.api.ProgramakerApi;

public class ProgramakerBridgeService extends Service {
    public static final String BridgeUserNotificationChannel = "PROGRAMAKER_BRIDGE_USER_NOTIFICATION";
    public static final CharSequence BridgeUserNotificationChannelName = "User notifications";
    private static final long WAIT_TIME_BEFORE_RESTART_MILLIS = 10000; // 10s
    private ProgramakerAndroidBridge bridge = null;
    private static final String LogTag = "PM BridgeService";

    @Override
    public void onCreate() {
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Toast.makeText(this, "Starting bridge", Toast.LENGTH_SHORT).show();
        connectBridge();

        // If we get killed, after returning from here, restart
        return START_STICKY;
    }

    private void connectBridge() {
        ConfigManager config = new ConfigManager(this);

        String token = config.getToken();
        if (token == null) {
            Toast.makeText(this, "Cannot start bridge", Toast.LENGTH_SHORT).show();
            if (token == null) {
                Log.e(LogTag, "Cannot start bridge: Token is null");
            }
        }

        if (ProgramakerBridgeService.this.bridge != null) {
            // Toast.makeText(this, "Bridge already started", Toast.LENGTH_SHORT).show();
            Log.w(LogTag, "Bridge already started (not null)");
        }

        // Start up the thread running the service. Note that we create a
        // separate thread because the service normally runs in the process's
        // main thread, which we don't want to block. We also make it
        // background priority so CPU-intensive work doesn't disrupt our UI.
        Thread thread = new Thread(() -> {
            try {
                ProgramakerApi api = new ProgramakerApi();
                api.setToken(token);
                String userId = api.getUserId();

                String bridgeIdCheck = config.getBridgeId();
                if (bridgeIdCheck == null) {
                    bridgeIdCheck = api.createBridge(ProgramakerAndroidBridge.GetBridgeName(this));
                    config.setBridgeId(bridgeIdCheck);
                }
                final String bridgeId = bridgeIdCheck;

                ProgramakerBridgeService.this.bridge = ProgramakerAndroidBridge.configure(
                        this,
                        userId,
                        bridgeId);
                ProgramakerBridgeService.this.bridge.start(
                        () -> { // On ready
                            if (config.getBridgeConnectionId() == null) {
                                new DoAsync().executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR,
                                        new Tuple2<>(
                                                () -> {
                                                    boolean established = api.establishConnection(bridgeId);
                                                    if (!established) {
                                                        Log.e(LogTag, "Error establishing connection to bridge");
                                                    }
                                                },
                                                ex -> {
                                                    Log.e(LogTag, "Error establishing bridge connection: " + ex, ex);
                                                    ProgramakerBridgeService.this.stopSelf();
                                                }
                                        )
                                );
                            }
                        },
                        () -> { // On completed
                            ProgramakerBridgeService.this.bridge = null;
                            onBridgeFailedAfterConnected();
                        });
            }
            catch (Throwable ex) {
                Log.e(LogTag, "Error on bridge", ex);
                ProgramakerBridgeService.this.bridge = null;
            }
        }, "ServiceStartArguments");
        thread.setPriority(Process.THREAD_PRIORITY_BACKGROUND);
        thread.start();
    }

    private void onBridgeFailedAfterConnected() {
        Log.e(LogTag, "Bridge stopped after connected. Waiting 10s then restarting");
        try {
            Thread.sleep(WAIT_TIME_BEFORE_RESTART_MILLIS);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        connectBridge();
    }

    @Override
    public IBinder onBind(Intent intent) {
        // We don't provide binding, so return null
        return null;
    }

    @Override
    public void onDestroy() {
        Toast.makeText(this, "Bridge stopped", Toast.LENGTH_SHORT).show();
    }
}
